import os, typing
from .sqlite import SqliteDB, TableNotFound  # expoted
from .legacy import *  # deprecated


class Table:
    def __init__(self, db: SqliteDB, table_name: str) -> None:
        self.db = db
        self.table_name = table_name

    def select(
        self,
        where: typing.Optional[str] = None,
        columns: typing.List[str] = ["*"],
        params={},
    ) -> list:
        return self.db.select(self.table_name, where, columns, params)

    def insert(self, values: typing.Dict[str, typing.Any] = {}):
        return self.db.insert(self.table_name, values)

    def update(self, where: str, set: typing.Dict[str, typing.Any], params):
        return self.db.update(self.table_name, where, set, params)

    def delete(self, where: str, params):
        return self.db.delete(self.table_name, where, params)


def get_db():
    db_url = os.getenv("ABSTRA_DATABASE_URL")
    if db_url.startswith("postgres://"):
        raise NotImplementedError("Postgres is not supported yet")
    elif db_url.startswith("mysql://"):
        raise NotImplementedError("MySQL is not supported yet")
    else:
        return SqliteDB(db_url)


def get_table(table_name: str):
    return Table(get_db(), table_name)



import io, tempfile, pathlib
from typing import Union

from .apis import upload_file
from .types import PILImage


def convert_file(file: Union[str, io.IOBase, PILImage]) -> str:
    if not file:
        return ""

    if isinstance(file, str):
        # URL or base64 encoded string
        if file.startswith("http") or file.startswith("data:"):
            return file

        # path to file
        return upload_file(open(file, "rb"))

    if isinstance(file, io.IOBase):
        return upload_file(file)

    # PILImage
    file_path = pathlib.Path(tempfile.gettempdir(), "img.png")
    file.save(str(file_path))
    file = open(file_path, "rb")
    return upload_file(file)


def download_file(url):
    import requests

    f = tempfile.NamedTemporaryFile()
    with requests.get(url, stream=True) as r:
        r.raise_for_status()
        for chunk in r.iter_content(chunk_size=8192):
            f.write(chunk)
    f.seek(0)
    return f



import flask, flask_sock, os

from ..api import API
from .utils import send_from_dist
from ..session import LiveSession, StaticSession
from ..runtimes import run_dash, run_form, run_hook


def __form_ws(api: API, session: LiveSession, form_path: str):
    form = api.get_form(form_path)
    if not form:
        session.close(reason="Not found")
        return

    code = api.read_text_file(form.file) if form.file else ""
    run_form(session, form, code)


def __dash_ws(api: API, session: LiveSession, dash_path: str):
    dash = api.get_dash(dash_path)
    if not dash:
        session.close(reason="Not found")
        return

    code = api.read_text_file(dash.file) if dash.file else ""
    run_dash(session, dash, code)


def get_player_bp(api: API):
    bp = flask.Blueprint("player", __name__)
    sock = flask_sock.Sock(bp)

    @bp.route("/_api/<path:id_or_path>", methods=["GET"])
    def get_runner_data(id_or_path):
        runtime = api.get_page_runtime(id_or_path)
        if not runtime:
            print("404", id_or_path)
            flask.abort(404)

        workspace = api.get_workspace()
        res = {}
        res[runtime.runner_type] = {
            **runtime.runner_dto,
            "workspace": workspace.runner_dto,  # TODO: fix this in frontend
        }
        return res

    @bp.route("/_version", methods=["GET"])
    def get_version():
        return os.getenv("ABSTRA_BUILD_ID")

    @sock.route("/_socket")
    def websocket(conn: flask_sock.Server):
        is_preview = flask.request.args.get("isPreview", False)
        session = LiveSession(conn, bool(is_preview))

        try:
            dash_path = flask.request.args.get("dashPath")
            if dash_path is not None:
                __dash_ws(api, session, dash_path)
                return

            form_path = flask.request.args.get("formPath")
            if form_path is not None:
                __form_ws(api, session, form_path)
                return

        finally:
            session.end()

    @bp.route("/_hooks/<path:path>", methods=["POST", "GET", "PUT", "DELETE", "PATCH"])
    def hook(path):
        hook_rt = api.get_hook(path)
        if not hook_rt:
            flask.abort(404)

        if not hook_rt.file:
            flask.abort(500)

        code = api.read_text_file(hook_rt.file)  # TODO: handle 404
        session = StaticSession()
        session.context["request"] = (
            flask.request.get_data(as_text=True),
            flask.request.args,
            flask.request.headers,
        )

        run_hook(code, session)

        body, status, headers = session.context.get("response", ({}, 200, {}))
        return flask.Response(status=status, headers=headers, response=body)

    @bp.route("/<path:filename>", methods=["GET"])
    def spa(filename: str):
        return send_from_dist(filename, "player.html")

    return bp



import unittest, tempfile
from abstra_server.runtimes.dashes.program import PythonProgram, DashPageState
from pathlib import Path
from .fixtures import init_dir
from abstra_server.api import API
from uuid import uuid4 as uuid
from datetime import datetime


class TestProgram(unittest.TestCase):
    def test_program_works(self):
        workspace_root_path = Path(tempfile.gettempdir(), f"{uuid()}")
        init_dir(workspace_root_path)
        api = API(root=workspace_root_path)
        dash = api.create_dash()
        code = "a = 1"
        py = PythonProgram(dash, code)
        py.execute_initial_code()
        a = py.ev("a")
        self.assertEqual(a, 1)

        # testing set variable
        py.set_variable("a", 2)
        a = py.ev("a")
        self.assertEqual(a, 2)
